# -*- coding: utf-8 -*-
"""
# @Time    : 2022/3/3 15:41
# @Author  : bruce
# @desc    :
"""
from api_test_ez.ez.decorator.singleton import singleton
from api_test_ez.ez.decorator.jsonbean import json_bean
